#ifndef ExtendedTask_H
#define ExtendedTask_H

#include <QStandardItem>

#include <State.hpp>
#include <simple_logger.hpp>

namespace hatptester{

class ExtendedTask: public QObject{ //Container that aggregates the informations from HATPState combined with values from interface
	Q_OBJECT

private:
	HATPState& state;
	const std::string taskName;
	QStandardItemModel model;
	
	ExtendedTask& operator=(const ExtendedTask&);
	
public slots:
	void planRequestUpdate(QStandardItem * item){
		if(item->column()!=1){//If the change occured in another column than value, the change is ingored
			return;
		}
		
		if(item->text().isEmpty()){//If the text of the cell is empty, the change is ignored as well
			return;
		}
		
		state.updateTask(taskName, item->model()->item(item->row(), 0)->text().toStdString(), item->text().toStdString());
		
	}
	
public:
	ExtendedTask(HATPState& state, const std::string& taskName):state(state), taskName(taskName){
		connect(&model, SIGNAL(itemChanged(QStandardItem*)), this, SLOT(planRequestUpdate(QStandardItem*)));
	
		std::pair<bool, const Task*> result=state.getTask(taskName);
		if(result.first){
			const Task& task=*result.second;
			
			model.setRowCount(task.getParameters().size());
		
			model.setColumnCount(2);
			QStringList labels;
			labels<<"Name"<<"Value";
			model.setHorizontalHeaderLabels(labels);

			int i=0;
			QStringList typeLabels;
			for(Task::Parameters::const_iterator it=task.getParameters().begin(); it!=task.getParameters().end(); ++it){
				std::string name=it->name;
				model.setItem(i, 0, new QStandardItem(name.c_str()));
				QStandardItem *item = new QStandardItem(it->value.c_str());
				item->setData(QVariant(QString(it->type.c_str())),Qt::UserRole);
				model.setItem(i, 1, item);
				typeLabels<<it->type.c_str();
				++i;
			}
			model.setVerticalHeaderLabels(typeLabels);			
		}else{
			LOG(error)<<"Tries to create a task from a name that doesn't exist in the system";
		}
	}
	
	ExtendedTask(const ExtendedTask& extended):state(extended.state), taskName(extended.taskName){
		connect(&model, SIGNAL(itemChanged(QStandardItem*)), this, SLOT(planRequestUpdate(QStandardItem*)));
		
		int rows=extended.model.rowCount();
		model.setColumnCount(2);
		model.setRowCount(rows);
		
		QStringList labels;
		labels<<"Name"<<"Value";
		model.setHorizontalHeaderLabels(labels);
		
		QStringList typeLabels;
		std::pair<bool, const Task*> result=state.getTask(taskName);
		if(result.first){
			const Task& task=*result.second;
			int i=0;
			QStringList typeLabels;
			for(Task::Parameters::const_iterator it=task.getParameters().begin(); it!=task.getParameters().end(); ++it){
				model.setItem(i, 0, new QStandardItem(extended.model.item(i, 0)->text()));
				QStandardItem *item = new QStandardItem(extended.model.item(i, 1)->text());
				item->setData(QVariant(extended.model.item(i, 1)->data(Qt::UserRole)),Qt::UserRole);
				model.setItem(i, 1, item);
				typeLabels<<it->type.c_str();
				++i;
			}
			model.setVerticalHeaderLabels(typeLabels);
		}else{
			LOG(error)<<"Tries to creates a task from a name that doesn't exist in the system";
		}
	}
	
	const std::string getName() const{
		return taskName;
	}

	QStandardItemModel* getParametersModel(){
		return &model;
	}
};

}; //End namespace

#endif //ExtendedTask_H
