#ifndef View_H
#define View_H

#include <iostream>

#include <QApplication>
#include <QMainWindow>
#include <QTextCodec>
#include <QMenuBar>
#include <QListView>
#include <QTableView>
#include <QLabel>
#include <QSpinBox>
#include <QCheckBox>
#include <QPushButton>
#include <QTimer>
#include <QGroupBox>
#include <QMessageBox>
#include <QGridLayout>
#include <QVBoxLayout>
#include <QHeaderView>
#include <QResizeEvent>
#include <QStatusBar>

#include <Controller.hpp>
#include <ParameterDelegate.hpp>
#include <simple_logger.hpp>

namespace hatptester{

class MainWindow : public QMainWindow{
	Q_OBJECT

private :
	MainWindow(const MainWindow&);
	MainWindow& operator=(const MainWindow&);

	HATPController& controller;

	QWidget centralPart;
	
	QGroupBox taskBox;
	QListView taskView;
	
	QGroupBox parameterBox;
	const std::string defaultText;
	QTableView parametersView;
	
	QLabel informationsText;
	QLabel statusBarContent;

	QAction quitAction;
	QAction sendFirstAction;
	QAction sendAllAction;

	QAction aboutAction;
	
	QCheckBox maxTimeToggle;
	QSpinBox maxTimeValue;
	bool changeToggle; //Represent the fact that the state changed, is used to compute if the button can be enabled
	QPushButton maxTimeButton;
	
	QPushButton sendFirstPlanRequestButton;
	QPushButton sendAllPlansRequestButton;
	
	QTimer timer; //This timer will be used to check every half a second that the client is still connected
	
	bool controllerWasReady;
	bool oldMaxTimeButton; //Rerpesent the previous state of the maxTimeButton
	bool oldParametersView; //Rerpesent the previous state of the parametersView
	void setAllButtonsDisable();
	void restoreAllButtonState();
		
public :
	MainWindow();
	
	void resizeEvent(QResizeEvent* event);
		
public slots:
	void about();
	
	void sendFirstPlanRequest();
	
	void sendAllPlansRequest();
	
	void activateSend();
	
	void updateInformations();
	
	void clicTask(const QModelIndex& index);
	
	void stateChanged();
	
	void periodicCheck();
	
	void maxTimeToggled(int);
	
	void maxTimeChanged(int);
	
	void sendMaxTime();
};

}; //namespace

#endif //View_H
